# bb_tl_heat.py weigu.lu
# This example is in the public domain
# Getting data from teensylogger and from
# Resol solar and heating controller
# Plotting data with gnuplot
# www.weigu.lu/b

# Import PyBBIO library:
from bbio import *
from time import gmtime, strftime, localtime
import shutil
import os
import shlex

A = GPIO1_17 #Header P9 Pin 23
B = GPIO1_16 #Header P9 Pin 15
T = GPIO3_21 #Header P9 Pin 25
Ser = Serial4

fpath = '/home/weigu/teensylogger/'
wpath = '/var/www/png/'
data_dir = fpath+'data/'
data_a_dir = fpath+'data_archive/'
png_dir = fpath+'png/'
gp_dir = fpath+'gp/'

pv_datafile1 = data_dir+'pv_data.min'
pv_datafile2 = data_dir+'pv_data_'
pv_gnupfile1 = gp_dir+'pv_gp_template.gp'
pv_gnupfile2 = gp_dir+'pv.gp'  

ec_datafile1 = data_dir+'ec_data.min'
ec_datafile2 = data_dir+'ec_data_'
ec_gnupfile1 = gp_dir+'ec_gp_template.gp'
ec_gnupfile2 = gp_dir+'ec.gp'

h_datafile1 = data_dir+'heating_data.min'
h_datafile2 = data_dir+'heating_data_'
h_gnupfile1 = gp_dir+'heating_gp_template.gp'
h_gnupfile2 = gp_dir+'heating.gp'  


ftime = strftime("%Y_%m_%d", localtime())
ftime2 = strftime("%d.%m.%y", localtime())
ftime3 = strftime("%d.%m.%y\t%H:%M:%S", localtime())

def get_teensylogger_pv(datafile1,datafile2):
    digitalWrite(A,LOW) # set gpios (multiplexing EIA232)
    digitalWrite(B,LOW)        
    delay(10)   
    Ser.begin(9600, timeout=0.1)
    Ser.write("g")
    delay(10)   
    #print "Data send: 'g'"
    data = ''                  
    for i in range(80): # get data
        char = Ser.read() 
        if char == "\r":
            break
        data += char
    data += "\n"        
    Ser.end()      #close serial port
    print 'pv:  ',data 
    #print "Connection closed!"
    pv_data=data[0:18] 
    pv1 = int(data[18]+data[19],16)*30    #2000 pulse/kWh  impulse *30 = power in Watt
    pv2 = int(data[21]+data[22],16)*30    
    pv3 = int(data[24]+data[25],16)*30    
    if pv1 == 30:
        pv1 = 0
    if pv2 == 30:
        pv2 = 0
    if pv3 == 30:
        pv3 = 0
    pve1 = 0
    pve2 = 0
    pve3 = 0
    try:
        f = open (datafile1, 'a+') #append + read
    except IOError:
        print ("Cannot create or find file: " + datafile1)
    try:    
        f2 = open (datafile2+ftime+'.min', 'a')
    except IOError:
        print ("Cannot create or find file: " + datafile2)    
    lineList = f.readlines()            #read all lines
    #print len(lineList)
    if (len(lineList)) == 0:
        pv_data = 'date time WP[W] WG[W] WK[W] WP[Wh] WG[Wh] WK[Wh]'+'\n'
    elif (len(lineList)) == 1:
        pv_data = pv_data+str(pv1)+' '+str(pv2)+' '+str(pv3)+' 0 0 0'+'\n'
    else:	
        line = lineList[len(lineList)-1]    #get the last line
        #print line
        lline =shlex.split(line)            #convert string (space seperated items) to list
        #print lline
        pve1 = (pv1/60)+int(lline[5]) #Wh
        pve2 = (pv2/60)+int(lline[6])
        pve3 = (pv3/60)+int(lline[7])  
        pv_data = pv_data+str(pv1)+' '+str(pv2)+' '+str(pv3)+' '+str(pve1)+' '+str(pve2)+' '+str(pve3)+'\n'
    f.write(pv_data)
    f2.write(pv_data)
    f.close()
    f2.close()
    delay(10)
            
def get_teensylogger_ec(datafile1,datafile2):
    digitalWrite(A,HIGH)	# set gpios (multiplexing EIA232)
    digitalWrite(B,LOW)        
    delay(10)   
    Ser.begin(9600, timeout=0.1)
    Ser.write("g")
    delay(10)   
    #print "Data send: 'g'"
    data = ''
    for i in range(80):
        char = Ser.read() 
        if char == "\r":
            break
        data += char
    data += "\n"    
    print 'ec:  ',data 
    Ser.end()      #close serial port
    #print "Connection closed!"
    ec_data=data[0:18] 
    ec1 = int(data[18]+data[19],16)*60    #1000 pulse/kWh  impulse *60 = power in Watt
    ec2 = int(data[21]+data[22],16)*60    
    ec3 = int(data[24]+data[25],16)*60    
    ec4 = int(data[27]+data[28],16)*60    
    ec5 = int(data[30]+data[31],16)*60    
    ec6 = int(data[33]+data[34],16)*60    
    ec7 = int(data[36]+data[37],16)*60    
    ec8 = int(data[39]+data[40],16)*60    
    try:
        f = open (datafile1, 'a+') #append + read
    except IOError:
        print ("Cannot create or find file: " + datafile1)
    try:    
        f2 = open (datafile2+ftime+'.min', 'a')
    except IOError:
        print ("Cannot create or find file: " + datafile2)    
    lineList = f.readlines()            #read all lines
    #print len(lineList)
    if (len(lineList)) == 0:
        ec_data = 'date time L1_old[W] L2_old[W] L3_old[W] L1_new[W] L2_new[W] L3_new[W] Leaf[W] heating[W]'+'\n'
    elif (len(lineList)) == 1:
        ec_data = ec_data+str(ec1)+' '+str(ec2)+' '+str(ec3)+' '+str(ec4)+' '+str(ec5)+' '+str(ec6)+' '+str(ec7)+' '+str(ec8)+'\n'
    else:	
        line = lineList[len(lineList)-1]    #get the last line
        lline =shlex.split(line)            #convert string (space seperated items) to list
        #print lline
        ec_data = ec_data+str(ec1)+' '+str(ec2)+' '+str(ec3)+' '+str(ec4)+' '+str(ec5)+' '+str(ec6)+' '+str(ec7)+' '+str(ec8)+'\n'
    f.write(ec_data)
    f2.write(ec_data)
    f.close()
    f2.close()
    delay(10)
    
def get_resol_heating(datafile1,datafile2):
    digitalWrite(A,HIGH) # set gpios (multiplexing EIA232)
    digitalWrite(B,HIGH)        
    digitalWrite(T,HIGH)        
    delay(10)     
    Ser.begin(9600, timeout=0.5)
    Ser.write(0x04)
    delay(10)
    #print "Data send: '0x04'"
    # Set the RTS pin.
    digitalWrite(T,LOW)        
    delay(100)
    data = []
    for i in range(70): # get data
        a = Ser.read()
        if a != '':
            data.append(ord(a))
    Ser.end()      #close serial port
    digitalWrite(T,HIGH)        
    print 'heating:   ',data        
    print len(data)
    #print "Connection closed!"
    # decoding: 60 Bytes
    # 2 Bytes per Databyte = High Nibbles are relevant! Low Nibbles
    # are fixed 7 and 6, ex.: 17 06 = 0x10 = 16
    # first byte = 0xB (0), Byte 2+3 = coded (0x1C = 28) Byte 4 = 0xC 
    # Data beginning at Byte 4 with date and time (4-15)
    # followed with 14*temp (16-43), LEDs (44-45), Relais (46-49) and burner
    # starts (50-53) and burner time (54-57) + 2 Bytes ?
    for i in range (4,len(data)):
        if (data[i]%16 == 7):
            data[i/2-2] = (data[i]/16)*16+(data[i+1]/16)
    for i in range (6,20): #temp
        if data [i] > 127:
            data [i] = (data [i] -256) +30
        else:    
            data [i] = data [i]+30
    for i in range (28,36): #8 LED    
        data[i] =  data[20]%2      
        data[20] = data[20]/2    
    for i in range (36,44): #8 relais
        data[i] =  data[22]%2      
        data[22] = data[22]/2    
    for i in range (44,50): #6 relais    
        data[i] =  data[21]%2      
        data[21] = data[21]/2    
    bs=data[24]*256+data[23]
    bl=data[26]*256+data[25]
    #print data        
    #print bs,bl            
    try:
        f = open (datafile1, 'a+') #append + read
    except IOError:
        print ("Cannot create or find file: " + datafile1)
    try:    
        f2 = open (datafile2+ftime+'.min', 'a')
    except IOError:
        print ("Cannot create or find file: " + datafile2)    
    lineList = f.readlines()            #read all lines
    #print len(lineList)
    hdata = ''
    if (len(lineList)) == 0:
        h_data = 'date\ttime\toutdoor[cent.]\tsolar_coll.[cent.]\t' \
        'boiler_hot_water[cent.]\tboiler_burner[cent.]\tboiler_solar[cent.]\t' \
        'burner_flow[cent.]\tburner_return[cent.]\tflow_A[cent.]\tflow_B' \
        '[cent.]\tset_flow_A[cent.]\tset_flow_B[cent.]\tLED_A_auto\tLED_A_day' \
        '\tLED_A_night\tLED_B_auto\tLED_B_day\tLED_B_night\tLED_eco\tLED_hot_' \
        'water\trelay_(R)_burner1\tR_burner2\tR_pump_A\tR_mixer_A_open\t' \
        'R_mixer_A_close\tR_zone_valve_flow\tR_zone_valve_return\tR_pump_sola' \
        'r\tR_pump_B\tR_mixer_B_open\tR_mixer_B_close\tR_pump_solid_fuel\tR_' \
        'pump circ\tR sec pump burner\tburner_starts\tburner_time[h]\tburner_' \
        'time [min]\tdate_(SH3214)\ttime_(SH3214)'+'\n'
    else:   
        h_data = ftime3+'\t'+str(data[6])+'\t'+str(data[12])+'\t'+str(data[9]) \
        +'\t'+str(data[14])+'\t'+str(data[13])+'\t'+str(data[10])+'\t' \
        +str(data[15])+'\t'+str(data[7])+'\t'+str(data[8])+'\t'+str(data[18])\
        +'\t'+str(data[19])+'\t'+str(data[28])+'\t'+str(data[29])+'\t' \
        +str(data[30])+'\t'+str(data[31])+'\t'+str(data[32])+'\t'+str(data[33])\
        +'\t'+str(data[34])+'\t'+str(data[35])+'\t'+str(data[36])+'\t' \
        +str(data[37])+'\t'+str(data[38])+'\t'+str(data[39])+'\t'+str(data[40])\
        +'\t'+str(data[41])+'\t'+str(data[42])+'\t'+str(data[43])+'\t' \
        +str(data[44])+'\t'+str(data[45])+'\t'+str(data[46])+'\t'+str(data[47])\
        +'\t'+str(data[48])+'\t'+str(data[49])+'\t'+str(bs)+'\t'+str(bl) \
        +'\t'+str(data[0])+'\t'+str(data[1])+'\t'+str(data[2])+'\t' \
        +str(data[3])+'\t'+str(data[4])+'\t'+str(data[5])+'\n'
    print h_data
    f.write(h_data)
    f2.write(h_data)
    f.close()
    f2.close()
    delay(10)
            
def pv_plot_file():
    Title = ftime2
    XFormat = "'%H:%M'"
    XTics = '60*60' #seconds
    Begin = ftime2 +' 05:00:00'
    End = ftime2 +' 23:00:00'
    Output = png_dir+'pv_'+ftime+'.png'
    Input = pv_datafile1
    try:
        os.remove(pv_gnupfile2)
    except OSError:
        pass
    try:
        gf1 = open (pv_gnupfile1,'r')
    except IOError:
        print ("Cannot find file: " + pv_gnupfile1)
    try:
        gf2 = open (pv_gnupfile2,'a')
    except IOError:
        print ("Cannot find file: " + pv_gnupfile2)    
    gline1 = gf1.readline()
    while gline1 != '':
        if '%TITLE%' in gline1:
            gline1 = gline1.replace('%TITLE%',Title)
        if '%XFORMAT%' in gline1:
            gline1 = gline1.replace('%XFORMAT%',XFormat)
        if '%XTICS%' in gline1:
            gline1 = gline1.replace('%XTICS%',XTics)
        if '%BEGIN%' in gline1:
            gline1 = gline1.replace('%BEGIN%',Begin)
        if '%END%' in gline1:
            gline1 = gline1.replace('%END%',End)
        if '%OUTPUT%' in gline1:
            gline1 = gline1.replace('%OUTPUT%',Output)
        if '%INPUT%' in gline1:
            gline1 = gline1.replace('%INPUT%',Input)
        gf2.write(gline1)
        gline1 = gf1.readline()
    gf1.close
    gf2.close

def ec_plot_file():
    Title = ftime2
    XFormat = "'%H:%M'"
    XTics = '60*60' #seconds
    Begin = ftime2 +' 00:00:00'
    End = ftime2 +' 24:00:00'
    Output = png_dir+'ec_'+ftime+'.png'
    Input = ec_datafile1
    try:
        os.remove(ec_gnupfile2)
    except OSError:
        pass
    try:
        gf1 = open (ec_gnupfile1,'r')
    except IOError:
        print ("Cannot find file: " + ec_gnupfile1)
    try:
        gf2 = open (ec_gnupfile2,'a')
    except IOError:
        print ("Cannot find file: " + ec_gnupfile2)    
    gline1 = gf1.readline()
    while gline1 != '':
        if '%TITLE%' in gline1:
            gline1 = gline1.replace('%TITLE%',Title)
        if '%XFORMAT%' in gline1:
            gline1 = gline1.replace('%XFORMAT%',XFormat)
        if '%XTICS%' in gline1:
            gline1 = gline1.replace('%XTICS%',XTics)
        if '%BEGIN%' in gline1:
            gline1 = gline1.replace('%BEGIN%',Begin)
        if '%END%' in gline1:
            gline1 = gline1.replace('%END%',End)
        if '%OUTPUT%' in gline1:
            gline1 = gline1.replace('%OUTPUT%',Output)
        if '%INPUT%' in gline1:
            gline1 = gline1.replace('%INPUT%',Input)
        gf2.write(gline1)
        gline1 = gf1.readline()
    gf1.close
    gf2.close

def heating_plot_file():
    Title = ftime2
    XFormat = "'%H:%M'"
    XTics = '60*60' #seconds
    Begin = ftime2 +'\t00:00:00'
    End = ftime2 +'\t24:00:00'
    Output = png_dir+'heating_'+ftime+'.png'
    Input = h_datafile1
    try:
        os.remove(h_gnupfile2)
    except OSError:
        pass
    try:
        gf1 = open (h_gnupfile1,'r')
    except IOError:
        print ("Cannot find file: " + h_gnupfile1)
    try:
        gf2 = open (h_gnupfile2,'a')
    except IOError:
        print ("Cannot find file: " + h_gnupfile2)    
    gline1 = gf1.readline()
    while gline1 != '':
        if '%TITLE%' in gline1:
            gline1 = gline1.replace('%TITLE%',Title)
        if '%XFORMAT%' in gline1:
            gline1 = gline1.replace('%XFORMAT%',XFormat)
        if '%XTICS%' in gline1:
            gline1 = gline1.replace('%XTICS%',XTics)
        if '%BEGIN%' in gline1:
            gline1 = gline1.replace('%BEGIN%',Begin)
        if '%END%' in gline1:
            gline1 = gline1.replace('%END%',End)
        if '%OUTPUT%' in gline1:
            gline1 = gline1.replace('%OUTPUT%',Output)
        if '%INPUT%' in gline1:
            gline1 = gline1.replace('%INPUT%',Input)
        gf2.write(gline1)
        gline1 = gf1.readline()
    gf1.close
    gf2.close

def setup():
    pinMode(A, OUTPUT)  
    pinMode(B, OUTPUT)
    pinMode(T, OUTPUT)  

def loop():
    try:
        get_teensylogger_pv(pv_datafile1,pv_datafile2)
    except:
        print ('Cannot get data from teensylogger pv')
    try:        
        pv_plot_file ()
        os.system('/usr/bin/gnuplot '+pv_gnupfile2)
        shutil.copy(png_dir+'pv_'+ftime+'.png', wpath+'pv_daily.png')
    except:
        print ('Cannot plot pv data')    
    try:
        get_teensylogger_ec(ec_datafile1,ec_datafile2)
    except:
        print ('Cannot get data from teensylogger ec')
    try:                
        ec_plot_file ()
        os.system('/usr/bin/gnuplot '+ec_gnupfile2)
        shutil.copy(png_dir+'ec_'+ftime+'.png', wpath+'ec_daily.png')
    except:
        print ('Cannot plot pv data')
    try:
        get_resol_heating (h_datafile1,h_datafile2)
    except:
        print ('Cannot get data from resol controller')    
    heating_plot_file ()
    print 'A'
    os.system('/usr/bin/gnuplot '+h_gnupfile2)
    print 'B'
    shutil.copy(png_dir+'heating_'+ftime+'.png', wpath+'heating_daily.png')
    print 'C'
    #except:
    #    print ('Cannot plot heating data')

    stop()         #no loop 
  
run(setup, loop)
